﻿// Copyright (c) 2012 Morten Bakkedal
// This code is published under the MIT License.

using System;
using System.Collections.Generic;
using FuncLib.Collections;

namespace FuncLib.Functions // FIXME rename
{
	public class DependencySorter
	{
		private DependencySorter()
		{
		}

		public static T[] Sort<T>(params T[] items) where T : class, IDependency<T>
		{
			// Initialize by adding in arbitrary order and mark them all as unprocessed.
			DependencyList<T> sorted = new DependencyList<T>(items);
			LinkedList<T> unprocessed = new LinkedList<T>(items);

			while (unprocessed.Count > 0)
			{
				T item = unprocessed.First.Value;
				unprocessed.RemoveFirst();

				foreach (T dependencyItem in item.Dependencies)
				{
					// Add a dependency relation. The dependency item will the placed before the current item (and
					// moved if already in the list, but not in the right position).
					if (sorted.AddDependency(item, dependencyItem))
					{
						// Dependency item not seen before (i.e. the list was enlarged). Mark as unprocessed.
						unprocessed.AddLast(dependencyItem);
					}
				}
			}

			return sorted.ToArray();
		}
	}
}
