﻿// Copyright (c) 2012 Morten Bakkedal
// This code is published under the MIT License.

using System;
using System.Collections.Generic;
using System.Diagnostics;

using FuncLib.Functions.Compilation;

namespace FuncLib.Functions
{
	[Serializable]
	[DebuggerDisplay("{Constant}")]
	public class ConstantFunction : Function
	{
		public ConstantFunction(double constant)
		{
			Constant = constant;
		}

		protected override double ComputeValue(IEvaluator evaluator) // FIXME
		{
			return Constant;
		}

		protected override double ComputeValue(IValueEvaluator evaluator)
		{
			return Constant;
		}

		protected override Function ComputeDerivative(Variable variable)
		{
			return 0.0;
		}

		protected override Function ComputePartialValue(IPartialEvaluator evaluator)
		{
			return this;
		}

		public override Expression Compile(CodeGenerator generator)
		{
			return Constant;
		}

		/*public static bool TryCast(Function function, out double constant)
		{
			if (function is ConstantFunction)
			{
				constant = ((ConstantFunction)function).Constant;
				return true;
			}

			if (function.IsZero)
			{
				constant = 0.0;
				return true;
			}

			constant = double.NaN;
			return false;
		}*/

		public override IEnumerable<IFunction> Dependencies
		{
			get
			{
				// No dependencies.
				yield break;
			}
		}

		public double Constant
		{
			get;
			private set;
		}
	}
}
