﻿// Copyright (c) 2012 Morten Bakkedal
// This code is published under the MIT License.

using System;

namespace FuncLib.Functions
{
	public interface IEvaluator
	{
		double Value(Function function);

		void Add<TKey, TValue>(TKey key, TValue value);

		bool TryGetValue<TKey, TValue>(TKey key, out TValue value);

		IPoint Point
		{
			get;
		}
	}

	public interface IPointEvaluator<TKey, TValue> : IEvaluator<TKey, TValue>
	{
		IPoint Point
		{
			get;
		}
	}

	public class PreparedValueEvaluator
	{
		private IFunction[] functions, sortedFunctions;

		private PreparedValueEvaluator(IFunction[] functions)
		{
			this.functions = (IFunction[])functions.Clone();

			sortedFunctions = DependencySorter.Sort<IFunction>(functions);
		}

		public static PreparedValueEvaluator Create(IFunction[] functions)
		{
			throw new NotImplementedException();
		}

		public ValueEvaluator CreateEvaluator()
		{
			throw new NotImplementedException();
		}


		public double[] Evaluate(IPoint point)
		{
			throw new NotImplementedException();
		}
	}

	public class ValueEvaluator : IValueEvaluator
	{
		private System.Collections.Generic.Dictionary<IFunction, double> values;

		public ValueEvaluator(IPoint point)
		{
			Point = point;

			values = new System.Collections.Generic.Dictionary<IFunction, double>();
		}

		public static double Evaluate(IPoint point, IFunction function)
		{
			return Evaluate(point, new IFunction[] { function })[0];
		}

		public static double[] Evaluate(IPoint point, IFunction[] functions)
		{
			ValueEvaluator evaluator = new ValueEvaluator(point);

			foreach (IFunction function in DependencySorter.Sort<IFunction>(functions))
			{
				evaluator.values[function] = function.Value(evaluator);
			}

			double[] values = new double[functions.Length];
			for (int i = 0; i < functions.Length; i++)
			{
				values[i] = evaluator[functions[i]];
			}

			return values;
		}

		public IPoint Point
		{
			get;
			private set;
		}

		public double this[IFunction key]
		{
			get
			{
				return values[key];
			}
		}
	}

	public interface IValueEvaluator : IPointEvaluator<IFunction, double> // FIXME or just IEvaluator ??
	{
	}

	public interface XIPartialValueEvaluator : IPointEvaluator<IFunction, IFunction> // FIXME or just IEvaluator ??
	{
	}

	public interface GradientEval : IEvaluator<IFunction, GradientEval>
	{
	}

	public interface IEvaluator<TKey, TValue>
	{
		TValue this[TKey key]
		{
			get;
		}
	}
}
