// Copyright (c) 2012 Morten Bakkedal
// This code is published under the MIT License.

using System;
using System.Collections.Generic;
using System.Diagnostics;

namespace FuncLib.Functions
{
	[Serializable]
	[DebuggerStepThrough]
	public class VariableAssignment
	{
		public VariableAssignment(Variable variable, double value)
		{
			Variable = variable;
			Value = value;
		}

		public static VariableAssignment[] Create(IDictionary<Variable, double> assignments)
		{
			return new List<KeyValuePair<Variable, double>>(assignments).ConvertAll<VariableAssignment>(assignment => new VariableAssignment(assignment.Key, assignment.Value)).ToArray();
		}

		public static VariableAssignment[] Create(Variable[] variables, double[] values)
		{
			int n = variables.Length;

			if (values.Length != n)
			{
				throw new ArgumentException("Number number of variables and the number of values don't agree.");
			}

			List<VariableAssignment> assignments = new List<VariableAssignment>();
			for (int i = 0; i < n; i++)
			{
				assignments.Add(new VariableAssignment(variables[i], values[i]));
			}

			return assignments.ToArray();
		}

		public Variable Variable
		{
			get;
			private set;
		}

		public double Value
		{
			get;
			private set;
		}
	}
}
